// Includes necessary libraries and headers for WiFi, MQTT, and the Si115X sensor
#include <ESP8266WiFi.h>  // Library for WiFi connectivity
#include <MQTT.h>         // Library for MQTT communication
#include "conns.h"       // Custom connection details (SSID, password, broker, etc.)
#include "Si115X.h"      // Library for the Si115X sunlight sensor

// Initialize the Si115X sensor object
Si115X si1151;

// Create network and MQTT client objects
WiFiClient net;  // WiFi client object
MQTTClient client; // MQTT client object

unsigned long lastMillis = 0; // Stores last time a message was published

//___________________________________________________

// Function to establish WiFi and MQTT connection
void connect() {
  Serial.print("checking wifi...");
  while (WiFi.status() != WL_CONNECTED) { // Wait until connected to WiFi
    Serial.print(".");
    delay(1000);
  }

  Serial.print("\nconnecting...");
  while (!client.connect("CatPat", "", "")) { // Connect to the MQTT broker with an identifier
    Serial.print(".");
    delay(1000);
  }
  Serial.println("\nconnected!");
}

//__________________________________________________

// Setup function runs once when the device starts
void setup() {
  Serial.begin(115200); // Initialize serial communication
  WiFi.begin(ssid, pass); // Connect to WiFi using credentials from "conns.h"
  client.begin(myBroker, net); // Connect to the specified MQTT broker
  connect(); // Establish initial connection to WiFi and MQTT
  
  // Initialize the Si115X Sunlight Sensor
  if (!si1151.Begin()) { 
    Serial.println("Si1151 is not ready!");
    while (1) { // If the sensor fails to initialize, stay in a loop and print dots
      delay(1000);
      Serial.print(".");
    }
  } else {
    Serial.println("Si1151 is ready!");
  }
}

//___________________________________________________

// Loop function runs continuously
void loop() {
  client.loop(); // Maintain MQTT connection
  delay(10); // Small delay to improve WiFi stability

  // If the MQTT client gets disconnected, attempt to reconnect
  if (!client.connected()) {
    connect();
  }
  
  // Publish data every second
  if (millis() - lastMillis > 1000) {
    lastMillis = millis(); // Update the last publish timestamp
    
    // Read IR and Visible light levels from the Si115X sensor
    int irLevel = si1151.ReadIR();
    int visibleLight = si1151.ReadVisible();
    
    // Concatenate values into a single string for easier publishing and subscribing
    String outval = String(irLevel) + "," + String(visibleLight);
    Serial.println(outval);
    
    Serial.print("Publishing: ");
    client.publish("catpatersonemo/IR", outval); // Publish sensor readings to the MQTT topic
  }
}

